@@ -1,7 +1,8 @@
+from .checks import geo_query
 from .checks.base import BaseCheck
 from .checks.base import CheckLevel
 from .checks.base import FileExistsCheck
-from .checks.base import GeneralCheck
+from .checks.base import ForeignKeyCheck
 from .checks.base import NotNullCheck
 from .checks.base import QueryCheck
 from .checks.base import RangeCheck
@@ -21,11 +22,11 @@ from .checks.factories import generate_geometry_type_checks
 from .checks.factories import generate_not_null_checks
 from .checks.factories import generate_type_checks
 from .checks.factories import generate_unique_checks
-from .checks.other import BankLevelCheck
 from .checks.other import BoundaryCondition1DObjectNumberCheck
 from .checks.other import ConnectionNodesDistance
 from .checks.other import ConnectionNodesLength
 from .checks.other import CrossSectionLocationCheck
+from .checks.other import LinestringLocationCheck
 from .checks.other import OpenChannelsWithNestedNewton
 from .checks.other import Use0DFlowCheck
 from .checks.timeseries import TimeseriesIncreasingCheck
@@ -34,10 +35,7 @@ from .checks.timeseries import TimeseriesTimestepCheck
 from .checks.timeseries import TimeseriesValueCheck
 from .threedi_model import models
 from .threedi_model.models import constants
-from geoalchemy2 import functions as geo_func
-from sqlalchemy import and_
 from sqlalchemy import func
-from sqlalchemy import or_
 from sqlalchemy.orm import Query
 from typing import List
 
@@ -68,7 +66,6 @@ CHECKS += [
     RangeCheck(
         error_code=21,
         column=table.friction_value,
-        filters=table.friction_type == constants.FrictionType.CHEZY.value,
         min_value=0,
     )
     for table in [
@@ -81,8 +78,7 @@ CHECKS += [
     RangeCheck(
         error_code=21,
         column=table.friction_value,
-        filters=(table.friction_type == constants.FrictionType.CHEZY.value)
-        & (table.crest_type == constants.CrestType.BROAD_CRESTED.value),
+        filters=(table.crest_type == constants.CrestType.BROAD_CRESTED.value),
         min_value=0,
     )
     for table in [
@@ -93,11 +89,12 @@ CHECKS += [
 CHECKS += [
     RangeCheck(
         error_code=22,
+        level=CheckLevel.WARNING,
         column=table.friction_value,
         filters=table.friction_type == constants.FrictionType.MANNING.value,
-        min_value=0,
         max_value=1,
         right_inclusive=False,  # 1 is not allowed
+        message=f"{table.__tablename__}.friction_value is not less than 1 while MANNING friction is selected. CHEZY friction will be used instead. In the future this will lead to an error.",
     )
     for table in [
         models.CrossSectionLocation,
@@ -108,12 +105,13 @@ CHECKS += [
 CHECKS += [
     RangeCheck(
         error_code=23,
+        level=CheckLevel.WARNING,
         column=table.friction_value,
         filters=(table.friction_type == constants.FrictionType.MANNING.value)
         & (table.crest_type == constants.CrestType.BROAD_CRESTED.value),
-        min_value=0,
         max_value=1,
         right_inclusive=False,  # 1 is not allowed
+        message=f"{table.__tablename__}.friction_value is not less than 1 while MANNING friction is selected. CHEZY friction will be used instead. In the future this will lead to an error.",
     )
     for table in [
         models.Orifice,
@@ -154,11 +152,11 @@ CHECKS += [
                 ]
             ),
         ),
-        message=f"Channel.calculation_type cannot be "
+        message=f"v2_channel.calculation_type cannot be "
         f"{constants.CalculationType.EMBEDDED}, "
         f"{constants.CalculationType.CONNECTED} or "
         f"{constants.CalculationType.DOUBLE_CONNECTED} when "
-        f"GlobalSetting.dem_file is null",
+        f"v2_global_settings.dem_file is null",
     )
 ]
 
@@ -182,9 +180,11 @@ CHECKS += [
 CHECKS += [
     RangeCheck(
         error_code=43,
+        level=CheckLevel.WARNING,
         column=table.dist_calc_points,
         min_value=0,
         left_inclusive=False,  # 0 itself is not allowed
+        message=f"{table.__tablename__}.dist_calc_points is not greater than 0, in the future this will lead to an error",
     )
     for table in [models.Channel, models.Pipe, models.Culvert]
 ]
@@ -195,7 +195,7 @@ CHECKS += [
         invalid=Query(models.ConnectionNode)
         .join(models.Manhole)
         .filter(models.ConnectionNode.storage_area < 0),
-        message="The ConnectionNode.storage_area should be >= 0 when the ConnectionNode is a Manhole",
+        message="v2_connection_nodes.storage_area is not greater than or equal to 0",
     ),
 ]
 
@@ -203,18 +203,18 @@ CHECKS += [
 ## 005x: CROSS SECTIONS
 
 CHECKS += [
-    CrossSectionLocationCheck(max_distance=1.0, error_code=52),
+    CrossSectionLocationCheck(
+        level=CheckLevel.WARNING, max_distance=1.0, error_code=52
+    ),
     OpenChannelsWithNestedNewton(error_code=53),
     QueryCheck(
         error_code=54,
         column=models.CrossSectionLocation.reference_level,
         invalid=Query(models.CrossSectionLocation).filter(
-            models.CrossSectionLocation.bank_level != None,
             models.CrossSectionLocation.reference_level
             > models.CrossSectionLocation.bank_level,
         ),
-        message="CrossSectionLocation.reference_level should be below the CrossSectionLocation.bank_level "
-        "when CrossSectionLocation.bank_level is not null",
+        message="v2_cross_section_location.bank_level (if supplied) should be above the .reference_level",
     ),
     QueryCheck(
         error_code=55,
@@ -229,56 +229,53 @@ CHECKS += [
 ## 006x: PUMPSTATIONS
 
 CHECKS += [
-    GeneralCheck(
+    QueryCheck(
         error_code=61,
         column=models.Pumpstation.upper_stop_level,
-        criterion_valid=and_(
-            models.Pumpstation.upper_stop_level > models.Pumpstation.lower_stop_level,
-            models.Pumpstation.upper_stop_level > models.Pumpstation.start_level,
+        invalid=Query(models.Pumpstation).filter(
+            models.Pumpstation.upper_stop_level <= models.Pumpstation.start_level,
         ),
+        message="v2_pumpstation.upper_stop_level should be greater than v2_pumpstation.start_level",
     ),
-    GeneralCheck(
+    QueryCheck(
         error_code=62,
         column=models.Pumpstation.lower_stop_level,
-        criterion_valid=and_(
-            models.Pumpstation.lower_stop_level < models.Pumpstation.start_level,
-            models.Pumpstation.lower_stop_level < models.Pumpstation.upper_stop_level,
-        ),
-    ),
-    GeneralCheck(
-        error_code=63,
-        column=models.Pumpstation.start_level,
-        criterion_valid=and_(
-            models.Pumpstation.start_level > models.Pumpstation.lower_stop_level,
-            models.Pumpstation.start_level < models.Pumpstation.upper_stop_level,
+        invalid=Query(models.Pumpstation).filter(
+            models.Pumpstation.lower_stop_level >= models.Pumpstation.start_level,
         ),
+        message="v2_pumpstation.lower_stop_level should be less than v2_pumpstation.start_level",
     ),
     RangeCheck(
         error_code=64,
         column=models.Pumpstation.capacity,
         min_value=0,
     ),
-    GeneralCheck(
+    QueryCheck(
         error_code=65,
         level=CheckLevel.WARNING,
         column=models.Pumpstation.capacity,
-        criterion_invalid=models.Pumpstation.capacity == 0.0,
+        invalid=Query(models.Pumpstation).filter(models.Pumpstation.capacity == 0.0),
+        message="v2_pumpstation.capacity should be be greater than 0",
     ),
 ]
 
 ## 007x: BOUNDARY CONDITIONS
 
 CHECKS += [
-    # 1d boundary conditions cannot be connected to a pumpstation
-    GeneralCheck(
+    QueryCheck(
         error_code=71,
         column=models.BoundaryCondition1D.connection_node_id,
-        criterion_invalid=or_(
-            models.BoundaryCondition1D.connection_node_id
-            == models.Pumpstation.connection_node_start_id,
-            models.BoundaryCondition1D.connection_node_id
-            == models.Pumpstation.connection_node_end_id,
+        invalid=Query(models.BoundaryCondition1D).filter(
+            (
+                models.BoundaryCondition1D.connection_node_id
+                == models.Pumpstation.connection_node_start_id
+            )
+            | (
+                models.BoundaryCondition1D.connection_node_id
+                == models.Pumpstation.connection_node_end_id
+            ),
         ),
+        message="v2_1d_boundary_conditions cannot be connected to a pumpstation",
     ),
     # 1d boundary conditions should be connected to exactly 1 object
     BoundaryCondition1DObjectNumberCheck(error_code=72),
@@ -375,7 +372,6 @@ CHECKS += [
 
 ## 01xx: LEVEL CHECKS
 
-CHECKS += [BankLevelCheck(error_code=101)]
 CHECKS += [
     QueryCheck(
         level=CheckLevel.WARNING,
@@ -390,7 +386,7 @@ CHECKS += [
         .filter(
             table.invert_level_start_point < models.Manhole.bottom_level,
         ),
-        message=f"{table}.invert_level_start_point should be higher than or equal to Manhole.bottom_level. In the future, this will lead to an error.",
+        message=f"{table.__tablename__}.invert_level_start_point should be higher than or equal to v2_manhole.bottom_level. In the future, this will lead to an error.",
     )
     for table in [models.Pipe, models.Culvert]
 ]
@@ -408,7 +404,7 @@ CHECKS += [
         .filter(
             table.invert_level_end_point < models.Manhole.bottom_level,
         ),
-        message=f"{table}.invert_level_end_point should be higher than or equal to Manhole.bottom_level. In the future, this will lead to an error.",
+        message=f"{table.__tablename__}.invert_level_end_point should be higher than or equal to v2_manhole.bottom_level. In the future, this will lead to an error.",
     )
     for table in [models.Pipe, models.Culvert]
 ]
@@ -427,8 +423,8 @@ CHECKS += [
             models.Pumpstation.type_ == constants.PumpType.SUCTION_SIDE,
             models.Pumpstation.lower_stop_level <= models.Manhole.bottom_level,
         ),
-        message="Pumpstation.lower_stop_level should be higher than "
-        "Manhole.bottom_level. In the future, this will lead to an error.",
+        message="v2_pumpstation.lower_stop_level should be higher than "
+        "v2_manhole.bottom_level. In the future, this will lead to an error.",
     ),
     QueryCheck(
         level=CheckLevel.WARNING,
@@ -444,8 +440,8 @@ CHECKS += [
             models.Pumpstation.type_ == constants.PumpType.DELIVERY_SIDE,
             models.Pumpstation.lower_stop_level <= models.Manhole.bottom_level,
         ),
-        message="Pumpstation.lower_stop_level should be higher than "
-        "Manhole.bottom_level. In the future, this will lead to an error.",
+        message="v2_pumpstation.lower_stop_level should be higher than "
+        "v2_manhole.bottom_level. In the future, this will lead to an error.",
     ),
     QueryCheck(
         level=CheckLevel.WARNING,
@@ -457,8 +453,8 @@ CHECKS += [
                 [constants.CalculationTypeNode.CONNECTED]
             ),
         ),
-        message="Manhole.drain_level >= Manhole.bottom_level when "
-        "Manhole.calculation_type is CONNECTED. In the future, this will lead to an error.",
+        message="v2_manhole.drain_level >= v2_manhole.bottom_level when "
+        "v2_manhole.calculation_type is CONNECTED. In the future, this will lead to an error.",
     ),
     QueryCheck(
         level=CheckLevel.WARNING,
@@ -473,7 +469,7 @@ CHECKS += [
             ),
             models.Manhole.drain_level == None,
         ),
-        message="Manhole.drain_level cannot be null when using sub-basins (v2_global_settings.manhole_storage_area > 0) and no DEM is supplied.",
+        message="v2_manhole.drain_level cannot be null when using sub-basins (v2_global_settings.manhole_storage_area > 0) and no DEM is supplied.",
     ),
 ]
 
@@ -485,15 +481,7 @@ CHECKS += [
         error_code=202,
         level=CheckLevel.WARNING,
         column=table.id,
-        invalid=Query(table).filter(
-            geo_func.ST_Length(
-                geo_func.ST_Transform(
-                    table.the_geom,
-                    Query(models.GlobalSetting.epsg_code).limit(1).label("epsg_code"),
-                )
-            )
-            < 0.05
-        ),
+        invalid=Query(table).filter(geo_query.length(table.the_geom) < 0.05),
         message=f"Length of a {table} is very short (< 0.05 m). A length of at least 1.0 m is recommended.",
     )
     for table in [models.Channel, models.Culvert]
@@ -520,6 +508,10 @@ CHECKS += [
     )
     for table in [models.Orifice, models.Weir]
 ]
+CHECKS += [
+    LinestringLocationCheck(error_code=205, column=table.the_geom, max_distance=1)
+    for table in [models.Channel, models.Culvert]
+]
 
 
 ## 025x: Connectivity
@@ -548,7 +540,7 @@ CHECKS += [
                 )
             ),
         ),
-        message="This is an isolated ConnectionNode without connections. Connect it to either a pipe, "
+        message="This is an isolated connection node without connections. Connect it to either a pipe, "
         "channel, culvert, weir, orifice or pumpstation.",
     ),
     QueryCheck(
@@ -608,8 +600,8 @@ CHECKS += [
             models.GlobalSetting.dem_obstacle_height <= 0,
             models.GlobalSetting.dem_obstacle_detection == True,
         ),
-        message="GlobalSetting.dem_obstacle_height should be larger than 0 when "
-        "GlobalSetting.dem_obstacle_detection == True",
+        message="v2_global_settings.dem_obstacle_height should be larger than 0 when "
+        "v2_global_settings.dem_obstacle_detection == True",
     ),
     QueryCheck(
         error_code=303,
@@ -639,9 +631,11 @@ CHECKS += [
     ),
     RangeCheck(
         error_code=306,
+        level=CheckLevel.WARNING,
         column=models.GlobalSetting.dist_calc_points,
         min_value=0,
         left_inclusive=False,  # 0 itself is not allowed
+        message="v2_global_settings.dist_calc_points is not greater than 0, in the future this will lead to an error",
     ),
     RangeCheck(
         error_code=307,
@@ -707,6 +701,43 @@ CHECKS += [
         invalid=CONDITIONS["has_no_dem"].filter(models.GlobalSetting.epsg_code == None),
         message="v2_global_settings.epsg_code may not be NULL if no dem file is provided",
     ),
+    QueryCheck(
+        error_code=318,
+        level=CheckLevel.WARNING,
+        column=models.GlobalSetting.epsg_code,
+        invalid=CONDITIONS["has_dem"].filter(models.GlobalSetting.epsg_code == None),
+        message="if v2_global_settings.epsg_code is NULL, it will be extracted from the DEM later, however, the modelchecker will use ESPG:28992 for its spatial checks",
+    ),
+    QueryCheck(
+        error_code=319,
+        column=models.GlobalSetting.use_2d_flow,
+        invalid=CONDITIONS["has_no_dem"].filter(
+            models.GlobalSetting.use_2d_flow == True
+        ),
+        message="v2_global_settings.use_2d_flow may not be TRUE if no dem file is provided",
+    ),
+    QueryCheck(
+        error_code=320,
+        column=models.GlobalSetting.use_2d_flow,
+        invalid=Query(models.GlobalSetting).filter(
+            models.GlobalSetting.use_1d_flow == False,
+            models.GlobalSetting.use_2d_flow == False,
+        ),
+        message="v2_global_settings.use_1d_flow and v2_global_settings.use_2d_flow cannot both be FALSE",
+    ),
+    QueryCheck(
+        level=CheckLevel.WARNING,
+        error_code=321,
+        column=models.GlobalSetting.manhole_storage_area,
+        invalid=Query(models.GlobalSetting).filter(
+            models.GlobalSetting.manhole_storage_area > 0,
+            (
+                (models.GlobalSetting.use_2d_flow == True)
+                | (~is_none_or_empty(models.GlobalSetting.dem_file))
+            ),
+        ),
+        message="sub-basins (v2_global_settings.manhole_storage_area > 0) should only be used when there is no DEM supplied and there is no 2D flow",
+    ),
 ]
 
 ## 04xx: Groundwater, Interflow & Infiltration
@@ -1348,6 +1379,71 @@ CHECKS += [
     ]
 ]
 
+## 122x Structure controls
+
+CHECKS += [
+    ForeignKeyCheck(
+        error_code=1220,
+        column=models.ControlMeasureMap.object_id,
+        reference_column=models.ConnectionNode.id,
+        filters=models.ControlMeasureMap.object_type == "v2_connection_node",
+    )
+]
+CHECKS += [
+    ForeignKeyCheck(
+        error_code=1221,
+        column=control_table.target_id,
+        reference_column=models.Channel.id,
+        filters=control_table.target_type == "v2_channel",
+    )
+    for control_table in (models.ControlMemory, models.ControlTable)
+]
+CHECKS += [
+    ForeignKeyCheck(
+        error_code=1222,
+        column=control_table.target_id,
+        reference_column=models.Pipe.id,
+        filters=control_table.target_type == "v2_pipe",
+    )
+    for control_table in (models.ControlMemory, models.ControlTable)
+]
+CHECKS += [
+    ForeignKeyCheck(
+        error_code=1223,
+        column=control_table.target_id,
+        reference_column=models.Orifice.id,
+        filters=control_table.target_type == "v2_orifice",
+    )
+    for control_table in (models.ControlMemory, models.ControlTable)
+]
+CHECKS += [
+    ForeignKeyCheck(
+        error_code=1224,
+        column=control_table.target_id,
+        reference_column=models.Culvert.id,
+        filters=control_table.target_type == "v2_culvert",
+    )
+    for control_table in (models.ControlMemory, models.ControlTable)
+]
+CHECKS += [
+    ForeignKeyCheck(
+        error_code=1225,
+        column=control_table.target_id,
+        reference_column=models.Weir.id,
+        filters=control_table.target_type == "v2_weir",
+    )
+    for control_table in (models.ControlMemory, models.ControlTable)
+]
+CHECKS += [
+    ForeignKeyCheck(
+        error_code=1226,
+        column=control_table.target_id,
+        reference_column=models.Pumpstation.id,
+        filters=control_table.target_type == "v2_pumpstation",
+    )
+    for control_table in (models.ControlMemory, models.ControlTable)
+]
+
 
 class Config:
     """Collection of checks
@@ -1369,32 +1465,50 @@ class Config:
                 error_code=1,
             )
             self.checks += generate_unique_checks(model.__table__, error_code=2)
-            self.checks += generate_not_null_checks(model.__table__, error_code=3)
+            self.checks += generate_not_null_checks(
+                model.__table__,
+                error_code=3,
+                custom_level_map={
+                    "v2_grid_refinement.refinement_level": "warning",
+                    "v2_grid_refinement.the_geom": "warning",
+                    "v2_grid_refinement_area.refinement_level": "warning",
+                    "v2_grid_refinement_area.the_geom": "warning",
+                },
+            )
             self.checks += generate_type_checks(
                 model.__table__,
                 error_code=4,
                 custom_level_map={
-                    "sewerage": "INFO",
-                    "external": "INFO",
+                    "*.sewerage": "INFO",
+                    "v2_weir.external": "INFO",
                 },
             )
             self.checks += generate_geometry_checks(
                 model.__table__,
-                custom_level_map={"the_geom_linestring": "info"},
+                custom_level_map={
+                    "v2_connection_nodes.the_geom_linestring": "info",
+                    "v2_grid_refinement.the_geom": "warning",
+                    "v2_grid_refinement_area.the_geom": "warning",
+                    "v2_dem_average_area.the_geom": "warning",
+                    "v2_surface.the_geom": "warning",
+                    "v2_impervious_surface.the_geom": "warning",
+                },
                 error_code=5,
             )
             self.checks += generate_geometry_type_checks(
                 model.__table__,
-                custom_level_map={"the_geom_linestring": "info"},
+                custom_level_map={
+                    "v2_connection_nodes.the_geom_linestring": "info",
+                },
                 error_code=6,
             )
             self.checks += generate_enum_checks(
                 model.__table__,
                 error_code=7,
                 custom_level_map={
-                    "sewerage_type": "INFO",
-                    "zoom_category": "INFO",
-                    "material": "INFO",
+                    "*.zoom_category": "INFO",
+                    "v2_pipe.sewerage_type": "INFO",
+                    "v2_pipe.material": "INFO",
                 },
             )
 
